/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.util;

import com.sun.jersey.api.client.ClientResponse;
import org.junit.Ignore;
import org.junit.Test;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.*;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public abstract class RetrieveApiTestTemplate extends BaseTestTemplate {

    public RetrieveApiTestTemplate(RestDescriptor getDescriptor, RestDescriptor deleteDescriptor, RestDescriptor createDescriptor) {
        super(getDescriptor);
        this.createDescriptor = createDescriptor;
        this.deleteDescriptor = deleteDescriptor;
    }

    protected RestDescriptor createDescriptor;
    protected RestDescriptor deleteDescriptor;
    private String id = "retrievetest" + System.currentTimeMillis();

    @Override
    protected String getId() {
        return id;
    }

    @Override
    protected void deleteResource() throws Exception {
        deleteDescriptor.run(getId(), testUtils.getOpsAccessToken());
    }

    @Override
    protected void createResource() throws Exception {
        createDescriptor.run(getId(), testUtils.getOpsAccessToken());
    }

    @Override
    protected int expectedOkResponseCode() {
        return 200;
    }


    @Test
    public void should_return404_when_accessingDataThatDoesntExist() throws Exception {
        deleteResource();
        ClientResponse response = descriptor.run(getId(), testUtils.getOpsAccessToken());
        assertEquals(error(response.getEntity(String.class)), 404, response.getStatus());
    }

    @Test
    public void should_beAbleToRunApiInParallel() throws Exception {
        createResource();
        ExecutorService executor = Executors.newFixedThreadPool(10);
        List<Callable<ClientResponse>> tasks = new ArrayList<>();

        for (int i = 0; i < 10; i++) {
            Callable<ClientResponse> task = () -> {
                try {
                    return descriptor.run(getId(), testUtils.getOpsAccessToken());
                } catch (Exception ex) {
                    return null;
                }
            };
            tasks.add(task);
        }

        List<Future<ClientResponse>> responses = executor.invokeAll(tasks);
        executor.shutdown();
        executor.awaitTermination(10, TimeUnit.SECONDS);

        deleteResource();
        int sucessResponseCount = 0;
        int errorResponseCount = 0;
        for (Future<ClientResponse> future : responses) {
            if (future.get().getStatus() == expectedOkResponseCode())
                sucessResponseCount++;
            else
                errorResponseCount++;
        }

        assertEquals(error("Expected all successful responses. Actual " + sucessResponseCount), 10, sucessResponseCount);
        assertEquals(error("Unexpected error response returned"), 0, errorResponseCount);
    }
}
